/*
 - testi2c.c by Laurent NAVET <Laurent_Navet@yahoo.com>
 - based on libmoebus by Boris LESNER <boris@lesner.org>
 - Controler les SMEC sur le port //

This software is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; version 2 dated June, 1991.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this software; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA.

 */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <asm/io.h>
#include "moebus.h"

/*
#define DATA 0x0378
#define STATUS DATA+1
#define CONTROL DATA+2
*/

#define _TRACE_


/*-----------------------------------
The pinout for the parallel port is as follows:
Parallel port connector

25 PIN D-SUB FEMALE at the PC.

 Pin  Name   Dir   Description
 1  STROBE  [-->] Strobe
 2  D0      [-->] Data Bit 0
 3  D1      [-->] Data Bit 1
 4  D2      [-->] Data Bit 2
 5  D3      [-->] Data Bit 3
 6  D4      [-->] Data Bit 4
 7  D5      [-->] Data Bit 5
 8  D6      [-->] Data Bit 6
 9  D7      [-->] Data Bit 7
 10 ACK     [<--] Acknowledge
 11 BUSY    [<--] Busy
 12 PE      [<--] Paper End
 13 SEL     [<--] Select
 14 AUTOFD  [-->] Autofeed
 15 ERROR   [<--] Error
 16 INIT    [-->] Initialize
 17 SELIN   [-->] Select In
 18 GND     [---] Signal Ground
 19 GND     [---] Signal Ground
 20 GND     [---] Signal Ground
 21 GND     [---] Signal Ground
 22 GND     [---] Signal Ground
 23 GND     [---] Signal Ground
 24 GND     [---] Signal Ground
 25 GND     [---] Signal Ground

You connect the driver circuit to pint 18 and to
the data pins (2-9).
------------------------------------*/

#define  SET_SCL    & 0xFE       //  1111 1110  ==> SCL sur bit 0 de DATA ==> Pin 2
#define  CLR_SCL    | 0x01       //  0000 0001
#define  SET_SDA    & 0xFD       //  1111 1101  ==> SDA sur bit 1 de DATA ==> Pin 3
#define  CLR_SDA    | 0x02       //  0000 0010

/*
#define  IN_SDA     & 0x40
#define  IN_SCL     & 0x80
*/
typedef unsigned char byte;

byte in_patt(byte device);
void out_patt(byte device, byte dirs, byte patt);
void out_byte(byte o_byte);
byte in_byte(void);
byte nack(void);
void start(void);
void stop(void);
void low_SDA(void);
void high_SDA(void);
void low_SCL(void);
void high_SCL(void);
void printfb(int);
byte in_SDA(void);

int main(int argc, char *argv[])
{
	unsigned char valeur, device, dirs;
	device = 0x00;
	dirs = 0x80;
	
	if(argc > 2 || argc == 1)
	{
		printf("amibguous command arguments, openning /dev/parport0\n");
		open_port("/dev/parport0");
	}
	else
		open_port(argv[1]);

	while(1)
	{
	  printf("Entrez la donne  crire (hexa) (FF, FE, etc..)\n(00 pour quitter):\n");
	  scanf("%x", &valeur);
	  if(!valeur)
	    break;
	  out_patt(device, dirs, valeur);
	  printf("\n");
	  printfb(in_patt(device));
	}

	close_port();
	return 0;
}


byte in_patt(byte device) 
     /* fetches byte from 8574 at specified addresss */
{
   byte o_byte, i_patt;
#ifdef _TRACE_
   printf("start: ");
#endif
   start();             // debut de communication
   o_byte=0x40 | (device << 1) | 0x01;          // 0010 0001
#ifdef _TRACE_
   printf("\nadresse: ");
#endif
   out_byte(o_byte);

#ifdef _TRACE_
   printf("\nnack: ");
#endif
   nack();         

#ifdef _TRACE_
   printf("\nlecture: ");
#endif
   i_patt=in_byte();  // lecture sur le bus

#ifdef _TRACE_
   printf("\nstop: ");
#endif
   stop();            // fin de communication
   return(i_patt);
}

void out_patt(byte device, byte dirs, byte patt)
     /* outputs specified pattern on specified 8574.  Note that
     ** inputs specified by "dirs" are maintained at a logic one */

{
   byte o_byte;

#ifdef _TRACE_
   printf("start: ");
#endif
   start();

   o_byte = 0x40 | (device << 1);   // 0100 0000

#ifdef _TRACE_
   printf("\nadresse: ");
#endif
   out_byte(o_byte);  // envoie l'adresse du smec sur 7 bits + 0 pour dire qu'on crit  

#ifdef _TRACE_
   printf("\nnack: ");
#endif
   nack();     

   o_byte = patt | dirs;   // valeur | Ox80 soit 1111 xxxx ??? a quoi a sert
#ifdef _TRACE_
   printf("\nvaleur: ");
#endif
   out_byte(o_byte);        

#ifdef _TRACE_
   printf("\nnack: ");
#endif
   nack();  

#ifdef _TRACE_
   printf("\nstop: ");
#endif
   stop();   
}

void out_byte(byte o_byte)
/* shift out byte, beginning with most significant bit */
/* ecrit l'octet sur le bus en commencant par le bit de poids fort */ 
{
   int n;
   for(n=7; n>=0; n--)
   {      /* note SCL is low during transitions on SDA */
     if (((o_byte >>n) & 0x01) == 0)   
      {
	low_SDA();    
      }
      else
      {
         high_SDA();

      }
     /* un coup d'horloge */
      high_SCL();
      low_SCL();
   }
}

byte in_byte(void)
/* fetch byte, most significant byte first */
/* lit l'octet sur le bus en commencant par le bit de poids fort */ 
{
   byte i_byte=0x00;
   int n;

   high_SDA();
   for (n=0; n<8; n++)
   {
      high_SCL();
      i_byte=(i_byte << 1) | in_SDA();
     /* note inversion on least significant bit of control port */
      low_SCL();
   }
   return(i_byte);
}

byte in_SDA(void)   /* renvoie l'tat de SDA */
{
byte ack_bit;
unsigned char status;
//   if ((inb(STATUS)&0x40)!=0x40)
 read_status(&status); 
 if((status & 0x40) != 0x40) //pourquoi 0x40 et pas ((status | 0xFC) != 0xFC)  => a tester
   {
      ack_bit=0;
   }
   else
   {
      ack_bit=1;
   }
   return(ack_bit);
}

byte nack(void)
{
   byte ack_bit;
   high_SDA();    // met SDA et SLC  l'tat haut
   high_SCL();
   sleep(1);      // dort 1s
   ack_bit=in_SDA();  // puis regarde si SDA  l'tat haut 
   low_SCL();     // et remet SCL  l'tat bas
   return(ack_bit);
}

void start(void)
/* bring SDA high to low while SCL is high */
/* Dbut de communication */
{
   high_SDA();
   high_SCL();
   low_SDA();  /* bring SDA low while clock is high */
   low_SCL();
}

void stop(void)
/* bring SDA low to high while SCL is high */
/* Fin de communication */
{
   low_SDA();
   high_SCL();
   high_SDA();  /* bring SDA high while clock is high */
   low_SCL();
}

void low_SDA(void)
{
  unsigned char data;
  read_data(&data);
  
  write_data(data CLR_SDA);

#ifdef _TRACE_
   printf("d");
#endif
        /* inversion required on ls bit of control port */
}

void high_SDA(void) /* makes output high impedance */
{
  unsigned char data;
  read_data(&data);
  
  write_data(data SET_SDA);

#ifdef _TRACE_
   printf("D");
#endif
}

void low_SCL(void)
{
  unsigned char data;
  read_data(&data);
  
  write_data(data CLR_SCL);

#ifdef _TRACE_
   printf("c");
#endif
}

void high_SCL(void)
{
  unsigned char data;
  read_data(&data);
  
  write_data(data SET_SCL);

#ifdef _TRACE_
   printf("C");
#endif
}



void printfb(int x)   // pourquoi int x alors qu'on passe un byte !!!!
{
   int n;

   printf ("\noctet lu: ");
   for(n=0; n<8; n++)
   {
      if((x & 0x80) !=0)
      {
         printf("1");
      }
      else
      {
         printf("0");
      }
      if (n==3)
      {
         printf(" "); /* insert a space between nybbles */
      }
      x = x<<1;
   }
   printf("\n");
}
